package com.management.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.management.constant.ERPURLConstant;
import com.management.constant.URLConstant;
import com.management.entity.Filter;
import com.management.utils.CrmRequestUtil;
import com.management.utils.KingDeeUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.*;

@RestController
@RequestMapping("/api/frame")
@Slf4j
public class FrameContractController {

    private RestTemplate restTemplate = new RestTemplate();

    public static void main(String[] args) {
        new FrameContractController().syncFrameContract();
    }

    /**
     * 同步框架合同
     *
     * @return void
     * @Author weiloong_zhang
     */
    @PostMapping
    public void syncFrameContract() {
        CrmRequestUtil crmRequestUtil = new CrmRequestUtil();

        String[] syncStatus = {"option_pending_sync__c", "option_resync__c"};

        JSONObject frameContract = crmRequestUtil.getCRMList(Arrays.asList(
                //new Filter("IS", "erp_id__c", Arrays.asList("")),
                new Filter("EQ", "life_status", Arrays.asList("normal")),
                new Filter("IN", "sync_status__c", Arrays.asList(syncStatus)),
                //new Filter("EQ", "name", Arrays.asList("KJ2505200005")),
                new Filter("GT", "create_time", Arrays.asList("1748923200000"))
        ), "framework_contract__c");

        JSONObject frameContractRes = new JSONObject();

        try {
            frameContractRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, frameContract, JSONObject.class);
        } catch (RestClientException e) {
            e.printStackTrace();
        }

        if (!"success".equals(frameContractRes.getString("errorDescription")) || frameContractRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
            log.info("查询失败或者不存在符合条件的数据");
            return;
        }

        log.info("查询成功，继续向下执行");

        for (Object frameContractObj : frameContractRes.getJSONObject("data").getJSONArray("dataList")) {
            JSONObject frameContractData = JSON.parseObject(JSON.toJSONString(frameContractObj));

            System.out.println("当前正在处理的框架合同数据为：" + frameContractData);

            //开始截取关键数据
            String frameContractId = frameContractData.getString("_id");
            //合同名称
            String frameContractName = frameContractData.getString("name");
            //创建时间
            //String createTime = frameContractData.getString("create_time");
            String createTime = frameContractData.getString("last_modified_time");

            //开始查询是否已经集成过该合同
            JSONObject isLogRes = new JSONObject();

            try {
                isLogRes = restTemplate.getForObject("http://localhost:18085/Log/query/log_data?table=send_log_htxx&log_type=KJHT&dataId=" + frameContractId + "&mark=" + createTime, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if (!isLogRes.getJSONArray("data").isEmpty()) {
                log.info("当前数据已经执行过了，将不再向下执行");
                continue;
            }

            log.info("日志不存在，将继续向下执行");

            //开始封装数据
            Map frameContractReqMap = new HashMap<>();
            frameContractReqMap.put("billno", frameContractName);//合同编号
            frameContractReqMap.put("crmid", frameContractId);//合同编号

            //开始查询商机编号
            String businessId = frameContractData.getString("business_opportunity__c") != null ? frameContractData.getString("business_opportunity__c") : "";

            if (businessId == null || businessId.equals("")) {
                log.info("当前数据没有商机编号，将不再向下执行");
                continue;
            }

            JSONObject businessReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", Arrays.asList(businessId))
            ), "NewOpportunityObj");

            JSONObject businessRes = new JSONObject();

            businessRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, businessReq, JSONObject.class);

            String projectCode = "";
            if ("success".equals(businessRes.getString("errorDescription")) && !businessRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject businessData = businessRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                projectCode = businessData.getString("opportunity_id__c");
            }

            //frameContractReqMap.put("project_number", "SJ20250424-0033");//todo 项目号（商机编号）
            frameContractReqMap.put("project_number", projectCode);//todo 项目号（商机编号）
            frameContractReqMap.put("billtype_number", "conm_salcontract_BT_QD");//erp单据类型
            frameContractReqMap.put("zhybilltype", frameContractData.getString("contract_type__c") != null ? frameContractData.getString("contract_type__c") : "");//合同类型
            frameContractReqMap.put("billname", frameContractData.getString("project_name__c") != null ? frameContractData.getString("project_name__c") : "");//合同名称
            frameContractReqMap.put("type_number", "XSHT-QD03");//合同类型
            frameContractReqMap.put("contparties_number", frameContractData.getString("our_company_name__c") != null ? frameContractData.getString("our_company_name__c") : "");//合同主体
            frameContractReqMap.put("createorg_number", frameContractData.getString("our_company_name__c") != null ? frameContractData.getString("our_company_name__c") : "");//销售组织编码
            frameContractReqMap.put("org_number", frameContractData.getString("our_company_name__c") != null ? frameContractData.getString("our_company_name__c") : "");//销售组织编码
            frameContractReqMap.put("currency_number", "CNY");//本位币.货币代码
            frameContractReqMap.put("settlecurrency_number", "CNY");//结算币别.货币代码

            long signDate = frameContractData.getLong("signing_date__c");

            frameContractReqMap.put("biztime", signDate);//签订日期,单据日期
            frameContractReqMap.put("biztimebegin", signDate);//起始日期
            frameContractReqMap.put("biztimeend", signDate);//截止日期
            frameContractReqMap.put("party1st", frameContractData.getString("contract_unit_name__c__r") != null ? frameContractData.getString("contract_unit_name__c__r") : "");//甲方
            frameContractReqMap.put("party2nd", frameContractData.getString("our_company_name__c__r") != null ? frameContractData.getString("our_company_name__c__r") : "");//乙方
            frameContractReqMap.put("bizmode", "C");//业务模式 A:统谈统签, B:统谈分签,C:分谈分签

            //部门编码
            String deptCode = "";

            List deptIdList = frameContractData.getJSONArray("data_own_department");

            //开始查询部门编码
            JSONObject deptReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", deptIdList)
            ), "DepartmentObj");

            JSONObject deptRes = new JSONObject();

            try {
                deptRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, deptReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(deptRes.getString("errorDescription")) && !deptRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject deptData = deptRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                deptCode = deptData.getString("field_Vj6sf__c") != null ? deptData.getString("field_Vj6sf__c") : "";
            }

            frameContractReqMap.put("dept", deptCode);//归属部门

            //获取负责人编码
            String ownerCode = "";

            List ownerIdList = frameContractData.getJSONArray("owner");

            JSONObject ownerReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "user_id", ownerIdList)
            ), "PersonnelObj");

            JSONObject ownerRes = new JSONObject();

            try {
                ownerRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, ownerReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(ownerRes.getString("errorDescription")) && !ownerRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject ownerData = ownerRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                ownerCode = ownerData.getString("field_T1xid__c") != null ? ownerData.getString("field_T1xid__c") : "";
            }

            //frameContractReqMap.put("operator", "ZHY12025");//todo 负责人
            frameContractReqMap.put("operator", ownerCode);

            //合同单位
            String customerId = frameContractData.getString("contract_unit_name__c");

            //根据客户唯一id查询客户名称
            JSONObject customerReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", Arrays.asList(customerId))
            ), "AccountObj");

            JSONObject customerRes = new JSONObject();

            try {
                customerRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, customerReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if (!"success".equals(customerRes.getString("errorDescription")) || customerRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                log.info("客户名称查询失败或者不存在，直接跳过更新");
                continue;
            }

            JSONObject customerData = customerRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);

            frameContractReqMap.put("customer_number", customerData.getString("account_no"));//合同单位名称
            frameContractReqMap.put("reccustomer_number", customerData.getString("account_no"));//收获客户单位名称
            //frameContractReqMap.put("customer_number", "Cus-000007");//todo 合同单位名称
            frameContractReqMap.put("settlecustomer_number", frameContractData.getString("billing_customer_code__c") != null ? frameContractData.getString("billing_customer_code__c") : "");//结算客户
            frameContractReqMap.put("payingcustomer_number", frameContractData.getString("payment_customer_code__c") != null ? frameContractData.getString("payment_customer_code__c") : "");//付款客户

            //获取产品线

            frameContractReqMap.put("productline_number", frameContractData.getString("product_line__c"));//产品线

            //获取具体安装方式

            frameContractReqMap.put("installationmethod", frameContractData.getString("installation_method__c"));//安装方式

            //行业类型
            frameContractReqMap.put("industrytype_number", frameContractData.getString("industry_type__c"));//行业类型
            frameContractReqMap.put("province", frameContractData.getString("field_t3fgo__c__r"));//省
            frameContractReqMap.put("city", frameContractData.getString("field_ngwth__c__r"));//市
            frameContractReqMap.put("county", frameContractData.getString("field_K5p2C__c__r"));//县
            frameContractReqMap.put("station", frameContractData.getString("single_choice__c") != null ? frameContractData.getString("single_choice__c") : "");//局级
            frameContractReqMap.put("delivery", frameContractData.getString("ship_on_payment_received__c") != null ? frameContractData.getString("ship_on_payment_received__c") : "");//款到发货
            frameContractReqMap.put("projectsource", frameContractData.getString("project_source__c") != null ? frameContractData.getString("project_source__c") : "");//项目来源
            frameContractReqMap.put("salesmethod", frameContractData.getString("sales_method__c") != null ? frameContractData.getString("sales_method__c") : "");//销售方式

            Map erpFrameContractReqMap = new HashMap<>();
            erpFrameContractReqMap.put("data", Arrays.asList(frameContractReqMap));//请求体

            String accessToken = new KDTokenController().getKDAccessTokenTest();

            if (accessToken == null || accessToken.equals("")) {
                log.info("金蝶token为空或不存在");
                return;
            }
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //开始封装请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);
            headers.set("accessToken", accessToken);
            headers.set("Idempotency-Key", uuid);

            HttpEntity contractReqEntity = new HttpEntity(erpFrameContractReqMap, headers);

            System.out.println(JSON.parseObject(JSON.toJSONString(contractReqEntity)));

            //开始向金蝶发起请求
            String contractUrl = ERPURLConstant.ERP_URL + "/ierp/kapi/v2/f9w5/conm/conm_salcontract/savecontract";

            JSONObject contractRes = new JSONObject();

            try {
                contractRes = restTemplate.postForObject(contractUrl, contractReqEntity, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            //开始封装回写程序
            Map backReqMap = new HashMap();
            backReqMap.put("_id", frameContractId);
            backReqMap.put("dataObjectApiName", "framework_contract__c");

            //开始封装日志
            Map logMap = new HashMap<>();
            logMap.put("log_id", UUID.randomUUID().toString().replace("-", ""));
            logMap.put("log_type", "KJHT");
            logMap.put("syn_type", "0");
            logMap.put("data_name", frameContractName);
            logMap.put("data_id", frameContractId);
            logMap.put("mark", createTime);
            logMap.put("send_body", JSON.toJSONString(contractReqEntity));
            logMap.put("send_res", JSON.toJSONString(contractRes));
            logMap.put("tableName", "send_log_htxx");

            //判断是否执行成功
            if ("0".equals(contractRes.getString("errorCode"))) {
                //同步成功
                logMap.put("log_status", "0");
                logMap.put("res_body", "同步成功");
                backReqMap.put("erp_id__c", contractRes.getJSONObject("data").getJSONArray("result").getJSONObject(0).getString("id"));
                backReqMap.put("sync_status__c", "option_sync_success__c");
                backReqMap.put("response_info__c", "同步成功");

                //审核
                KingDeeUtils kingDeeUtils = new KingDeeUtils();
                kingDeeUtils.audit(contractRes.getJSONObject("data").getJSONArray("result").getJSONObject(0).getString("id"), "conm_salcontract");
            } else {
                //同步失败
                logMap.put("log_status", "1");
                logMap.put("res_body", "同步失败：" + contractRes.getString("message"));
                backReqMap.put("sync_status__c", "option_sync_failure__c");
                backReqMap.put("response_info__c", contractRes.getString("message"));
            }

            //开始回写
            //解锁
            if ("1".equals(frameContractData.getString("lock_status"))) {
                log.info("解锁");
                crmRequestUtil.unlockCRM(frameContractId, "framework_contract__c");
            }
            JSONObject backReq = crmRequestUtil.updateCRM(backReqMap);

            JSONObject backRes = new JSONObject();

            try {
                backRes = restTemplate.postForObject(URLConstant.UPDATE_CRM_CUSTOMIZE, backReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("回写结果为：" + backRes);

            //开始记录日志
            JSONObject logRes = new JSONObject();
            try {
                logRes = restTemplate.postForObject("http://localhost:18085/Log/insert/log_data", logMap, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("日志插入结果为：" + logRes);

        }
    }
}
